#!/bin/env bash

set -e

# pg_cron package needs to be isntalled to let the testcase work

SCR_DIR=$(cd $(dirname "${BASH_SOURCE:-$0}") >/dev/null 2>&1 && pwd)
PG_MAJOR_VARSION="${1:-14}"
PGPORT="504${PG_MAJOR_VARSION}"
PGBASE="${SCR_DIR}/${PG_MAJOR_VARSION}"
PGDATA="${PGBASE}/data"

BIN_DIR="/usr/lib/postgresql/${PG_MAJOR_VARSION}/bin"

export PGPORT
export PGDATA

trap "cleanup" EXIT

#
# Functions
#
msg() {
	echo 
	echo --------------------------------------------------------------------------------
	echo "${*}"
	echo --------------------------------------------------------------------------------
}

cleanup() {
	msg Stop the server
	"${BIN_DIR}/pg_ctl" -l "${PGBASE}/server_startup.log" stop -mf || msg Failed to stop the server
}

#
# Script main code
#

msg System information
uname -a

msg Stop the test cluster if running
"${BIN_DIR}/pg_ctl" stop -mf && msg Server was sopped

msg Remove "${PGDATA}"
rm -rf "${PGDATA}"

msg Create new cluster
"${BIN_DIR}/initdb" --locale=en_US.UTF8

msg Start the server
"${BIN_DIR}/pg_ctl" -l "${PGBASE}/server_startup.log" start

msg Update configuration - add pg_cron to shared_preload_libraries
psql postgres -c "ALTER SYSTEM SET shared_preload_libraries=pg_cron;"

msg Restart the server to apply configuration change
"${BIN_DIR}/pg_ctl" -l "${PGBASE}/server_startup.log" restart -mf


msg Create database objects for tests
psql postgres <<EOF
\echo PostgreSQL version information
SELECT version();

\echo Create databae roles (superuser needed to create extension)
CREATE ROLE user1 WITH LOGIN SUPERUSER PASSWORD 'x';
CREATE ROLE user2 WITH LOGIN SUPERUSER PASSWORD 'x';

\echo Connect as user1
\c postgres user1

\echo List extensions
\dx

\echo Show shared_preload_libraries parameter value
SHOW shared_preload_libraries;

\echo Create extension pg_cron while connected as user1
CREATE EXTENSION pg_cron;

\echo List extensions
\dx

\echo List the table extension initial privileges
SELECT * FROM pg_catalog.pg_init_privs WHERE objoid = 'cron.job'::regclass::oid;
EOF

msg Create database dump with extension created
pg_dump -s -d postgres -f "${PGBASE}"/dump_1.sql

msg Change extension objects ownership
psql postgres <<EOF

\echo Display extension table privileges
\dp cron.job

\echo Change ownership to another role
REASSIGN OWNED BY user1 TO user2;

\echo Display extension table privileges
\dp cron.job

\echo List the table extension initial privileges
select * from pg_catalog.pg_init_privs where objoid = 'cron.job'::regclass::oid;

EOF

msg Create database dump with user1 onwership changed to user2
pg_dump -s -d postgres -f "${PGBASE}"/dump_2.sql

msg Drop role user1
psql postgres <<EOF

\echo Drop role user1 which was used to create pg_cron extension
DROP ROLE user1;

\echo List the table extension initial privileges
select * from pg_catalog.pg_init_privs where objoid = 'cron.job'::regclass::oid;

\echo Display extension table privileges
\dp cron.job

EOF

msg Dump database after role user1 was dropped
pg_dump -s -d postgres -f "${PGBASE}"/dump_3.sql

msg Print DUMP differences before and after drop of role user1
diff -d "${PGBASE}"/dump_2.sql "${PGBASE}"/dump_3.sql

